CON
  _CLKMODE      = XTAL1 + PLL16X                        
  _XINFREQ      = 5_000_000

  NUM_OF_XOR_BITS = 1
  SHOW_XOR_BITS = 1
  GUESSES = 30

OBJ
  rndm:  "RealRandom"
  buttons: "Touch Buttons"

VAR
  byte  leds
  byte  xorval[8]
  
pub XorIQ | i,button,wrong

  buttons.start(clkfreq / 100)
  rndm.start

  dira[16..23] := $ff

  repeat
    ' initialize the start-value 
    ' for a even xor-bitcount we need an even number of bits
    leds := ((getRandomNum // 4) + 2) & $fe
    ' for a odd number we need an odd number of bits 
    if (NUM_OF_XOR_BITS & 1) == 0
      leds--
    ' get a random set of bits (8 means there is no fixed bit)
    leds := getRandomBits( leds, 8 )

    ' for each button find random xor-pattern with the button bit fixed
    repeat i from 0 to 7
      xorval[i] := getRandomBits( NUM_OF_XOR_BITS, i )

    ' 
    outa[23..16] := leds
    repeat 9
      waitcnt( clkfreq>>3+cnt )
      outa[23..16] := 0
      waitcnt( clkfreq>>3+cnt )
      outa[23..16] := leds

    if SHOW_XOR_BITS
      repeat i from 0 to 7
        outa[23..16] := xorval[i]
        waitcnt( clkfreq>>2 + cnt )

    wrong := -1
    ' game loop
    repeat until leds==$ff or wrong==GUESSES
      wrong++
      ' set the output according to the current state
      outa[23..16] := leds

      ' this will wait until all buttons are released
      waitRelease
      ' this waits for exactly one button to be pushed
      button := getButton

      ' xor the current value with the random bit-pattern
      leds ^= xorval[ button ]

    if leds==$ff
      ' win - just a little blink
      outa[23..16] := $ff
      repeat 9
        waitcnt( clkfreq>>2+cnt )
        outa[23..16] := 0
        waitcnt( clkfreq>>2+cnt )
        outa[23..16] := $ff

      ' and a ping pong
      repeat 5
        leds:=1
        outa[23..16]:=leds
        repeat 7
          waitcnt( clkfreq>>3 + cnt )
          leds<<=1
          outa[23..16]:=leds
          
        repeat 7
          waitcnt( clkfreq>>3 + cnt )
          leds>>=1
          outa[23..16]:=leds
    
    else
      ' lost - faster blink
      repeat 50
        waitcnt( clkfreq>>4+cnt )
        outa[23..16] := 0
        waitcnt( clkfreq>>4+cnt )
        outa[23..16] := $ff
    

pub waitRelease
  repeat while buttons.State

  
pub getButton

  repeat
    result := buttons.State
    case result
      1:
        return 0
      2:
        return 1
      4:
        return 2
      8:
        return 3
      16:
        return 4
      32:
        return 5
      64:
        return 6
      128: 
        return 7
  

pub getRandomBits( bitcnt, fixbit ) | val,oldval,j
  ' if fixbit is in range 0-7 set the given bit
  if fixbit<8
    val:=|<fixbit
  ' if fixbit is 8 then initialize val
  else
    val:=0
  j:=0

  ' try to find a random bit until the bitcnt is satisfied
  repeat until j==bitcnt
    ' remember the old value
    oldval := val
    ' first get a random bit number
    val:=rndm.random & $07
    ' and set the bit in the bitfield that's already there
    val := |<val | oldval
    ' value changed, so a new bit has been found
    if oldval<>val
      j++
      
  return val
    
pub getRandomNum
  result := 0

  ' don't return 0 or $ff
  repeat until result and result<>$ff
    result := rndm.random & $ff
    
  return
  